package eu.eudat.file.transformer.rda.mapper;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import eu.eudat.file.transformer.models.descriptiontemplate.definition.FieldFileTransformerModel;
import eu.eudat.file.transformer.rda.DatasetId;
import eu.eudat.file.transformer.utils.json.JsonSearcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DatasetIdRDAMapper {
	private static final Logger logger = LoggerFactory.getLogger(DatasetIdRDAMapper.class);

	/*public static DatasetId toRDA(UUID id) {
		DatasetId rda = new DatasetId();
		rda.setIdentifier(id.toString());
		rda.setType(DatasetId.Type.OTHER);

		return rda;
	}*/

	public static DatasetId toRDA(List<FieldFileTransformerModel> nodes) {
		DatasetId data = new DatasetId();
		for (FieldFileTransformerModel node: nodes) {
			String rdaProperty = node.getSchematics().stream().filter(schematic -> schematic.startsWith("rda.dataset.dataset_id")).findFirst().orElse("");
			if (node.getData() == null) {
				continue;
			}
			String rdaValue = node.getData().getValue();
			if(rdaValue == null || rdaValue.isEmpty()){
				continue;
			}

			ObjectMapper mapper = new ObjectMapper();
			try {
				Map<String, Object> values = mapper.readValue(rdaValue, HashMap.class);
				if (!values.isEmpty()) {
					values.entrySet().forEach(entry -> finalRDAMap(data, entry.getKey(), (String) entry.getValue()));
				} else {
					finalRDAMap(data, rdaProperty, rdaValue);
				}
			} catch (IOException e) {
				logger.warn(e.getMessage() + ".Passing value as is");
				finalRDAMap(data, rdaProperty, rdaValue);

			}

		}

		if (data.getIdentifier() != null && data.getType() != null) {
			return data;
		}
		return null;
	}

	private static void finalRDAMap(DatasetId rda, String property, String value) {
		if (value != null) {
			for (DatasetIdProperties datasetIdProperties : DatasetIdProperties.values()) {
				if (property.contains(datasetIdProperties.getName())) {
					switch (datasetIdProperties) {
						case IDENTIFIER:
							rda.setIdentifier(value);
							break;
						case TYPE:
							try {
								rda.setType(DatasetId.Type.fromValue(value));
							}
							catch (IllegalArgumentException e){
								logger.warn("Type " + value + " from semantic rda.dataset.dataset_id.type was not found. Setting type to OTHER.");
								rda.setType(DatasetId.Type.OTHER);
							}
							break;
					}
				}
			}
		}


	}

	//TODO
/*
	public static List<Field> toProperties(DatasetId rda, JsonNode node) {
		List<Field> properties = new ArrayList<>();

		List<JsonNode> idNodes = JsonSearcher.findNodes(node, "schematics", "rda.dataset.dataset_id");

		for (JsonNode idNode: idNodes) {
			for (DatasetIdProperties datasetIdProperties : DatasetIdProperties.values()) {
				JsonNode schematics = idNode.get("schematics");
				if(schematics.isArray()){
					for(JsonNode schematic: schematics){
						if(schematic.asText().endsWith(datasetIdProperties.getName())){
							switch (datasetIdProperties) {
								case IDENTIFIER:
									Field field1 = new Field();
									field1.setKey(idNode.get("id").asText());
									field1.setValue(rda.getIdentifier());
									properties.add(field1);
									break;
								case TYPE:
									Field field2 = new Field();
									field2.setKey(idNode.get("id").asText());
									field2.setValue(rda.getType().value());
									properties.add(field2);
									break;
							}
							break;
						}
					}
				}
			}
		}
		return properties;
	}
*/

	private enum DatasetIdProperties {
		IDENTIFIER("identifier"),
		TYPE("type");

		private final String name;

		DatasetIdProperties(String name) {
			this.name = name;
		}

		public String getName() {
			return name;
		}
	}
}
