package eu.eudat.file.transformer.rda.mapper;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.node.ArrayNode;
import eu.eudat.file.transformer.models.descriptiontemplate.definition.FieldFileTransformerModel;
import eu.eudat.file.transformer.rda.TechnicalResource;
import eu.eudat.file.transformer.utils.string.MyStringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.*;
import java.util.stream.Collectors;

public class TechnicalResourceRDAMapper {
	private static final Logger logger = LoggerFactory.getLogger(TechnicalResourceRDAMapper.class);

	public static List<TechnicalResource> toRDAList(List<FieldFileTransformerModel> nodes) {
		Map<String, TechnicalResource> rdaMap = new HashMap<>();

		for (FieldFileTransformerModel node: nodes) {
			String rdaProperty = node.getSchematics().stream().filter(schematic -> schematic.startsWith("rda.dataset.technical_resource")).findFirst().orElse("");
			if (node.getData() == null) {
				continue;
			}
			String rdaValue = node.getData().getValue();
			if(rdaValue == null || rdaValue.isEmpty()){
				continue;
			}
			TechnicalResource rda = getRelative(rdaMap, node.getNumbering());
			if (!rdaMap.containsValue(rda)) {
				rdaMap.put(node.getNumbering(), rda);
			}
			for (ExportPropertyName exportPropertyName : ExportPropertyName.values()) {
				if (rdaProperty.contains(exportPropertyName.getName())) {
					switch (exportPropertyName) {
						case NAME:
							rda.setName(rdaValue);
							rda.setAdditionalProperty(ImportPropertyName.NAME.getName(), node.getId());
							break;
						case DESCRIPTION:
							rda.setDescription(rdaValue);
							rda.setAdditionalProperty(ImportPropertyName.DESCRIPTION.getName(), node.getId());
							break;
					}
				}
			}
		}

		return rdaMap.values().stream()
				.filter(tr -> tr.getName() != null)
				.collect(Collectors.toList());
	}

	//TODO
/*
	public static List<Field> toProperties(List<TechnicalResource> rdas) {
		List<Field> properties = new ArrayList<>();

		rdas.forEach(rda -> rda.getAdditionalProperties().entrySet().forEach(entry -> {
			try {
				Field field = new Field();
				field.setKey(entry.getValue().toString());
				ImportPropertyName importPropertyName = ImportPropertyName.fromString(entry.getKey());
				switch(importPropertyName) {
					case DESCRIPTION:
						field.setValue(rda.getDescription());
						break;
					case NAME:
						field.setValue(rda.getName());
						break;
				}
				properties.add(field);
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
			}
		}));

		return properties;
	}
*/

	public static TechnicalResource toRDA(JsonNode node) {
		TechnicalResource rda = new TechnicalResource();
		String rdaProperty = "";
		JsonNode schematics = node.get("schematics");
		if(schematics.isArray()){
			for(JsonNode schematic: schematics){
				if(schematic.asText().startsWith("rda.dataset.technical_resource")){
					rdaProperty = schematic.asText();
					break;
				}
			}
		}
		String value = node.get("value").asText();

		if (rdaProperty.contains("description")) {
			rda.setDescription(value);
		}
		if (rdaProperty.contains("name")) {
			rda.setName(value);
		}
		
		if (rda.getName() == null) {
			throw new IllegalArgumentException("Technical Resources Name is missing");
		}

		return rda;
	}

	private static TechnicalResource getRelative(Map<String, TechnicalResource> rdaMap, String numbering) {
		return rdaMap.entrySet().stream().filter(entry -> MyStringUtils.getFirstDifference(entry.getKey(), numbering) > 0)
					 .max(Comparator.comparingInt(entry -> MyStringUtils.getFirstDifference(entry.getKey(), numbering))).map(Map.Entry::getValue).orElse(new TechnicalResource());
	}

	private enum ExportPropertyName {
		NAME("name"),
		DESCRIPTION("description");

		private String name;

		ExportPropertyName(String name) {
			this.name = name;
		}

		public String getName() {
			return name;
		}
	}

	private enum ImportPropertyName {
		NAME("nameId"),
		DESCRIPTION("descriptionId");

		private String name;

		ImportPropertyName(String name) {
			this.name = name;
		}

		public String getName() {
			return name;
		}

		public static ImportPropertyName fromString(String name) throws Exception {
			for (ImportPropertyName importPropertyName: ImportPropertyName.values()) {
				if (importPropertyName.getName().equals(name)) {
					return importPropertyName;
				}
			}
			throw new Exception("Property name not available");
		}
	}
}
